@@ -1,6 +1,10 @@
 package Reddit::Client;
 
-our $VERSION = '1.35';  # 10/02/19 1.35 add_approved_user, minor housekeeping
+our $VERSION = '1.352'; # 1.352 10/25/19: iscomment, ispost and get_type are now static 
+# added functions distinguish, undistinguish
+
+# 10/05/19 1.351 delete now returns result
+#our $VERSION = '1.35';  # 10/02/19 1.35 add_approved_user, minor housekeeping
 # 1.341 7/30 removed warnings, they're stupid
 # 7/30 mute and unmute 
 # 1.33 7/10 corrected 'edited' to not be boolean
@@ -8,14 +12,9 @@ our $VERSION = '1.35';  # 10/02/19 1.35 add_approved_user, minor housekeeping
 # 5/3 .315 submit_comment now returns fullname not id
 # 4/25 .314 4/8 1.313
 #  .314 added locked key to Comment, was this a recent Reddit change?
-# 1.313 changed the behavior or print_request_errors
+# 1.313 changed the behavior of print_request_errors
 # 1.312 requests that fail with print_request_errors as true now die instead of
 # 		croak, which lets you capture the error message
-# 1.31 added a bunch of flags to Link, Thing, Comment, SubReddit
-#	   get_user now accepts 'username' as well as 'user'
-# 	   TODO: make this an alias across more functions, like sub/subreddit is
-
-# TODO: make ispost, iscomment and get_type static
 
 $VERSION = eval $VERSION;
 
@@ -130,6 +129,8 @@ use constant API_BY_ID               => 41;
 use constant API_FLAIR               => 42;
 use constant API_DELETEFLAIR         => 43;
 use constant API_UNBAN               => 44;
+use constant API_DISTINGUISH         => 45;
+use constant API_UNDISTINGUISH       => 46;
 
 #===============================================================================
 # Parameters
@@ -188,6 +189,8 @@ $API[API_BY_ID         ] = ['GET',  '/by_id'                  ];
 $API[API_FLAIR         ] = ['POST', '/r/%s/api/flair'         ];
 $API[API_DELETEFLAIR   ] = ['POST', '/r/%s/api/deleteflair'   ];
 $API[API_UNBAN         ] = ['POST', '/r/%s/api/unfriend'      ];
+$API[API_DISTINGUISH   ] = ['POST', '/api/distinguish'        ];
+$API[API_UNDISTINGUISH ] = ['POST', '/api/distinguish'        ];
 
 #===============================================================================
 # Class methods
@@ -225,6 +228,7 @@ sub new {
 		 croak "param 'user_agent' is required.";
 	}
 	$self->{user_agent} 	= $param{user_agent};
+	# request_errors does nothing?
 	$self->{request_errors} = $param{print_request_errors} || $param{request_errors} || 0;
 	$self->{print_response} = $param{print_response} || $param{print_response_conent} || 0;
 	$self->{print_request}  = $param{print_request} || 0;
@@ -837,19 +841,19 @@ sub ban {
 		$data->{note} = $param{note};
 	}
 
-	# $data->{container} not needed unless mode is friend or enemy
-	if ($param{duration}){
+	if ($param{duration}){ # if 0 this never even hits which we want anyway
 		if ($param{duration} > 999) {
 			print "Warning: Max duration is 999. Setting to 999.\n";
 			$param{duration} = 999;
 		} elsif ($param{duration} < 1) {
-			print "Warning: min duration is 1. Setting to indefinite.\n";
 			$param{duration} = 0;
 		}
 		$data->{duration} = $param{duration} if $param{duration};
 	}
+	# $data->{container} is not needed unless mode is friend or enemy
 	# $data->{permissions} = ?
-	# one of (friend, moderator, moderator_invite, contributor, banned, muted, wikibanned, wikicontributor)
+
+	# type is one of (friend, moderator, moderator_invite, contributor, banned, muted, wikibanned, wikicontributor)
 	if ($param{mode} eq 'mute') {
 		$data->{type} = 'muted';
 	} elsif ($param{mode} eq 'contributor') {
@@ -915,6 +919,63 @@ sub unmute {
 	return $self->unban(%param);
 }
 
+sub distinguish {
+	my ($self, $fullname, %param) = @_;
+	my $data = {};
+
+	if (!iscomment($fullname) and !ispost($fullname)) {
+		die 'Fullname is required (comment preceeded by "t1_", post "t3_")';
+	}
+
+	if (iscomment($fullname)) {
+		# only top level can be sticky
+		my $sticky = exists $param{sticky} ? $param{sticky} : 0;
+		$data->{sticky} = $sticky ? 'true' : 'false';
+	}
+
+	$data->{id} = $fullname;
+
+
+	$data->{how} = 'yes';
+	# Check manual setting of 'how'. Normal users should never set 'how'.
+	if ($param{how}) {
+		my @valid = qw/yes no admin special/;
+		my $ok;
+		for (@valid) {
+			if ($param{how} eq $_) {
+				$ok = 1;
+				last; # because we have to save potentially TWO CYCLES, right asshole? yeah spend all day on 2 cycles, that's a good use of your time
+			}
+		}
+
+		die "valid values for 'how' are: yes, no, admin, special\n" unless $ok;
+	}
+	
+	my $result = $self->api_json_request(
+		api  => API_DISTINGUISH, 
+		data => $data,
+	);
+	return $result;
+}
+
+sub undistinguish {
+	my ($self, $fullname, %param) = @_;
+	my $data = {};
+
+	if (!iscomment($fullname) and !ispost($fullname)) {
+		die 'Fullname is required (comment preceeded by "t1_", post "t3_")';
+	}
+
+	$data->{id} = $fullname;
+	$data->{how} = 'no';
+
+	my $result = $self->api_json_request(
+		api  => API_UNDISTINGUISH, 
+		data => $data,
+	);
+	return $result;
+}
+
 sub get_modlinks {
     my ($self, %param) = @_;
 
@@ -1073,7 +1134,7 @@ sub delete {
     	DEBUG('Delete post/comment %s', $name);
 
     	my $result = $self->api_json_request(api => API_DEL, data => { id => $name });
-    	return 1;
+    	return $result;
 }
 
 #===============================================================================
@@ -1220,7 +1281,7 @@ sub set_post_flair {
 	my $flairid	= $param{flair_template_id} || croak "need 'flair template id'";
 	my $data	= {};
 
-	if (!$self->ispost($post_id)) { $post_id = "t3_".$post_id; }
+	if (!ispost($post_id)) { $post_id = "t3_".$post_id; }
 	$data->{link} = $post_id;
 	$data->{flair_template_id} = $flairid;
 
@@ -1266,7 +1327,7 @@ sub get_flair_options {
 	my $data	= {};
 
 	if ($post_id) {
-		if (!$self->ispost($post_id)) { $post_id = "t3_".$post_id; }
+		if (!ispost($post_id)) { $post_id = "t3_".$post_id; }
 		$data->{link} = $post_id;
 	} elsif ($user) {
 		$data->{user} = $user;
@@ -1435,7 +1496,7 @@ sub submit_comment {
 	# the replies option, it does nothing
     #my $replies = exists $param{inbox_replies} ? ($param{inbox_replies} ? "true" : "false") : "true";
 
-    croak '$fullname must be a post or comment' if !$self->ispost($parent_id) && !$self->iscomment($parent_id);
+    croak '$fullname must be a post or comment' if !ispost($parent_id) && !iscomment($parent_id);
     DEBUG('Submit comment under %s', $parent_id);
 
     my $result = $self->api_json_request(api => API_COMMENT, data => {
@@ -1485,7 +1546,7 @@ sub vote {
     my ($self, $name, $direction) = @_;
     defined $name      || croak 'Expected $name';
     defined $direction || croak 'Expected $direction';
-    croak '$fullname must be a post or comment' if !$self->ispost($name) && !$self->iscomment($name);
+    croak '$fullname must be a post or comment' if !ispost($name) && !iscomment($name);
     croak 'Invalid vote direction' unless "$direction" =~ /^(-1|0|1)$/;
     DEBUG('Vote %d for %s', $direction, $name);
     $self->api_json_request(api => API_VOTE, data => { dir => $direction, id  => $name });
@@ -1498,7 +1559,7 @@ sub vote {
 sub save {
     my $self = shift;
     my $name = shift || croak 'Expected $fullname';
-    croak '$fullname must be a post or comment' if !$self->ispost($name) && !$self->iscomment($name);
+    croak '$fullname must be a post or comment' if !ispost($name) && !iscomment($name);
     DEBUG('Save %s', $name);
     $self->api_json_request(api => API_SAVE, data => { id => $name });
 }
@@ -1506,7 +1567,7 @@ sub save {
 sub unsave {
     my $self = shift;
     my $name = shift || croak 'Expected $fullname';
-    croak '$fullname must be a post or comment' if !$self->ispost($name) && !$self->iscomment($name);
+    croak '$fullname must be a post or comment' if !ispost($name) && !iscomment($name);
     DEBUG('Unsave %s', $name);
     $self->api_json_request(api => API_UNSAVE, data => { id => $name });
 }
@@ -1514,7 +1575,7 @@ sub unsave {
 sub hide {
     my $self = shift;
     my $name = shift || croak 'Expected $fullname';
-    croak '$fullname must be a post' if !$self->ispost($name);
+    croak '$fullname must be a post' if !ispost($name);
     DEBUG('Hide %s', $name);
     $self->api_json_request(api => API_HIDE, data => { id => $name });
 }
@@ -1522,7 +1583,7 @@ sub hide {
 sub unhide {
     my $self = shift;
     my $name = shift || croak 'Expected $fullname';
-    croak '$fullname must be a post' if !$self->ispost($name);
+    croak '$fullname must be a post' if !ispost($name);
     DEBUG('Unhide %s', $name);
     $self->api_json_request(api => API_UNHIDE, data => { id => $name });
 }
@@ -1651,20 +1712,22 @@ sub fullname {
 	return $id;
 }
 
-sub ispost { # todo: make this static function
-	my ($self, $name) = @_;
-    	my $type = substr $name, 0, 2;
+sub ispost { 
+	my $name = shift;
+	#my ($self, $name) = @_;
+    my $type = substr $name, 0, 2;
 	return $type eq 't3';
 }
 
-sub iscomment { # todo: make this static
-	my ($self, $name) = @_;
-    	my $type = substr $name, 0, 2;
+sub iscomment {
+	my $name = shift;
+	#my ($self, $name) = @_;
+    my $type = substr($name, 0, 2);
 	return $type eq 't1';
 }
-sub get_type { # ditto
-	my ($self, $name) = @_;
-    	return lc substr $name, 0, 2;
+sub get_type { 
+	my $name = shift;
+    return lc substr($name, 0, 2) if $name;
 }
 sub DEBUG {
     if ($DEBUG) {
@@ -1707,7 +1770,7 @@ Reddit::Client - A Perl wrapper for the Reddit API.
 
 Reddit::Client handles Oauth session management and HTTP communication with Reddit's external API. For more information about the Reddit API, see L<https://github.com/reddit/reddit/wiki/API>. 
 
-=head1 SYNOPSIS
+=head1 EXAMPLE
 
     use Reddit::Client;
     
@@ -1761,7 +1824,7 @@ Reddit::Client uses Oauth to communicate with Reddit. To get Oauth keys, visit y
 
 =item Script apps
 
-Most users will want a "script"-type app. This is an app intended for personal use that uses a username and password to authenticate. The I<description> and I<about url> fields can be empty, and the I<redirect URI> can be any valid URL (script apps don't use them). Once created, you can give other users permission to use it by adding them in the "add developer" field. Each account uses its own username and password to authenticate.
+Most new users will want a "script"-type app. This is an app intended for personal use that uses a username and password to authenticate. The I<description> and I<about url> fields can be empty, and the I<redirect URI> can be any valid URL (script apps don't use them). Once created, you can give other users permission to use it by adding them in the "add developer" field. Each account uses its own username and password to authenticate.
 
 Use the app's client id and secret along with your username and password to create a L<new|https://metacpan.org/pod/Reddit::Client#new> Reddit::Client object.
 
@@ -1769,7 +1832,7 @@ Use the app's client id and secret along with your username and password to crea
 
 As of v1.20, Reddit::Client also supports "web" apps. These are apps that can take actions on behalf of any user that grants them permission. (This uses the familiar "SomeRedditApp wants your permission to..." screen.)
 
-While they are fully supported, there is not yet a setup guide, so getting one running is left as an exercise for the reader. You will need a web server, which you will use to direct users to Reddit's authorization page, where the user will be asked to grant the app permissions. It will then redirect the user back to the app's redirect URI. This process generates a refresh token, which is a unique string that your app will use to authenticate instead of a username and password. You will probably want to store refresh tokens locally, otherwise you will have to get permission from the user every time the app runs.
+While they are fully supported, there is not yet a setup guide, so getting one running is left as an exercise for the reader. You will need a web server, which you will use to direct users to Reddit's authorization page, where the user will be asked to grant the app permissions. Reddit's authorization page will then redirect the user back to the app's redirect URI. This process generates a refresh token, which is a unique string that your app will use to authenticate instead of a username and password. You will probably want to store refresh tokens locally, otherwise you will have to get permission from the user every time the app runs.
 
 Documentation for the web app flow can be found at L<https://github.com/reddit-archive/reddit/wiki/OAuth2>.
 
@@ -1915,6 +1978,20 @@ Delete a post or comment.
 	
 Delete a multireddit.
 
+=item distinguish
+
+    distinguish ( $fullname,  [ sticky => 0, ] [ how => 'yes' ] )
+
+Distinguish a comment or post (moderator action). Fullname with prefix is required. Options:
+
+=over
+
+C<sticky> Distinguish and sticky a comment. Only works for top-level comments.
+
+C<how> This option should typically be left untouched. Valid values are "yes", "no", "admin", "special". Admin is for Reddit admins only, and special is for "special" users, which the API leaves unexplained.
+
+=back
+
 =item edit
 
     edit ( $fullname, $text )
@@ -2363,6 +2440,12 @@ Submit a text post. Returns the fullname of the new post. If C<inbox_replies> is
 
 Un-ban a user (moderator action).
 
+=item undistinguish
+
+    undistinguish ( $fullname )
+
+Un-distinguish a comment or post (moderator action). Fullname with prefix is required.
+
 =item unhide 
 
     unhide ( $fullname )
